#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "backend-operations.h"


void
on_file_new_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_file_open_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_file_save_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_file_save_as_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_file_quit_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_edit_preferences_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *preference_dialog;
	GtkWidget *entry_mimms_path;
	gchar *path_to_mimms;
	gboolean stop_dialog = FALSE;

	preference_dialog = create_preference_dialog();

	entry_mimms_path = lookup_widget(GTK_WIDGET(preference_dialog), "entry_mimms_path");
	path_to_mimms = get_mimms_path();
	gtk_entry_set_text(GTK_ENTRY(entry_mimms_path), path_to_mimms);

	while(stop_dialog != TRUE)
	{
		switch(gtk_dialog_run(GTK_DIALOG(preference_dialog)))
		{
			case GTK_RESPONSE_OK:
				/* set new path to mimms */
				path_to_mimms = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_mimms_path)));
				set_mimms_path(path_to_mimms);

				/* save preferences*/
				preference_save();

				/* stop dialog */
				stop_dialog = TRUE;
				break;

			case GTK_RESPONSE_APPLY:
				/* set new path to mimms */
				path_to_mimms = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_mimms_path)));
				set_mimms_path(path_to_mimms);

				/* save preferences*/
				preference_save();

				/* don't stop dialog */
				stop_dialog = FALSE;
				break;

			case GTK_RESPONSE_CANCEL:
				/* same as click on the dialog close button (default) */
			default:
				/* cancel dialog */
				stop_dialog = TRUE;
				break;
		}
	}
	gtk_widget_destroy(preference_dialog);
}


void
on_help_info_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *dialog;

	dialog = create_aboutdialog();

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}


void
on_checkbutton_use_recording_time_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *label_minutes;
	GtkWidget *spinbutton_minutes;

	label_minutes = lookup_widget(GTK_WIDGET(togglebutton), "label_recording_minutes");
	spinbutton_minutes = lookup_widget(GTK_WIDGET(togglebutton), "spinbutton_recording_minutes");

	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton)))
	{
		gtk_widget_set_sensitive(label_minutes, TRUE);
		gtk_widget_set_sensitive(spinbutton_minutes, TRUE);

		/* enable timer */
		mimms_use_timer(1);
	}
	else
	{
		gtk_widget_set_sensitive(label_minutes, FALSE);
		gtk_widget_set_sensitive(spinbutton_minutes, FALSE);

		/* disable timer */
		mimms_use_timer(0);
	}
}


void
on_entry_url_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gchar *converted_string;

	converted_string = g_locale_from_utf8(gtk_entry_get_text(GTK_ENTRY(editable)), -1, NULL, NULL, NULL);
	mimms_set_url(converted_string, GTK_WIDGET(editable));
	g_free(converted_string);
}


void
on_button_paste_url_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *entry_url;

	entry_url = lookup_widget(GTK_WIDGET(button), "entry_url");
	/* delete old text from entry field */
	gtk_editable_delete_text(GTK_EDITABLE(entry_url), 0, -1);
	/* paste clipboard to entry field */
	gtk_editable_paste_clipboard(GTK_EDITABLE(entry_url));
}


void
on_entry_save_as_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
	mimms_set_output_file(gtk_editable_get_chars(editable, 0, -1));
}


void
on_button_choose_file_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fileDialog;
	gint result;
	gchar *output_file;
	GtkWidget *entry_save_as;

	/* create and run a file chooser dialog */
	fileDialog = gtk_file_chooser_dialog_new(_("Save as"), NULL,
											GTK_FILE_CHOOSER_ACTION_SAVE,
											GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
											GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
											NULL);

	result = gtk_dialog_run(GTK_DIALOG(fileDialog));
	switch(result)
	{
		case GTK_RESPONSE_ACCEPT:
			/* get the path and filename for the output file */
			output_file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fileDialog));
			if(output_file == NULL)
			{
				return;
			}

			/* set entry_save_as to output_file */
			entry_save_as = lookup_widget(GTK_WIDGET(button), "entry_save_as");
			gtk_entry_set_text(GTK_ENTRY(entry_save_as), output_file);
			g_free(output_file);
			break;

		default:
			/* do nothing */
			break;
	}

	gtk_widget_destroy(fileDialog);
}


void
on_spinbutton_recording_minutes_value_changed
                                        (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
	GtkAdjustment *adjustment;

	adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spinbutton));
	mimms_set_timer(gtk_adjustment_get_value(adjustment));
}


void
on_button_start_download_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	/* check if file allready exists and open a warning dialog */
	/* ---- */
	start_download(GTK_WIDGET(button));
}


void
on_radiobutton_mms_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	mimms_set_method(mms_default, GTK_WIDGET(button), GTK_WIDGET_HAS_FOCUS(button));
}


void
on_radiobutton_mmst_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	mimms_set_method(mms_tcp, GTK_WIDGET(button), GTK_WIDGET_HAS_FOCUS(button));
}


void
on_radiobutton_mmsu_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	mimms_set_method(mms_udp, GTK_WIDGET(button), GTK_WIDGET_HAS_FOCUS(button));
}


void
on_radiobutton_mmsh_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	mimms_set_method(mms_http, GTK_WIDGET(button), GTK_WIDGET_HAS_FOCUS(button));
}


void
on_radiobutton_http_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	mimms_set_method(mms_asx_http, GTK_WIDGET(button), GTK_WIDGET_HAS_FOCUS(button));
}

void
on_entry_mimms_path_changed            (GtkEditable     *editable,
                                        gpointer         user_data)
{

}


void
on_button_mimms_path_autosearch_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *path_to_mimms;
	GtkWidget *entry_mimms_path;

	entry_mimms_path = lookup_widget(GTK_WIDGET(button), "entry_mimms_path");

	path_to_mimms = find_mimms_binary();
	if(path_to_mimms == NULL)
	{
		return;
	}

	gtk_entry_set_text(GTK_ENTRY(entry_mimms_path), path_to_mimms);
	g_free(path_to_mimms);
}


void
on_button_mimms_path_manualsearch_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fileDialog;
	gint result;
	gchar *path_to_mimms;
	GtkWidget *entry_mimms_path;

	/* create and run a file chooser dialog */
	fileDialog = gtk_file_chooser_dialog_new(_("Select MiMMS binary"), NULL,
											GTK_FILE_CHOOSER_ACTION_OPEN,
											GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
											GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
											NULL);
	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(fileDialog), TRUE);

	result = gtk_dialog_run(GTK_DIALOG(fileDialog));
	switch(result)
	{
		case GTK_RESPONSE_ACCEPT:
			/* get the path and filename from the dialog */
			path_to_mimms = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fileDialog));
			if(path_to_mimms == NULL)
			{
				return;
			}

			/* set entry_mimms_path to path_to_mimms */
			entry_mimms_path = lookup_widget(GTK_WIDGET(button), "entry_mimms_path");
			gtk_entry_set_text(GTK_ENTRY(entry_mimms_path), path_to_mimms);
			g_free(path_to_mimms);
			break;

		default:
			/* do nothing */
			break;
	}

	gtk_widget_destroy(fileDialog);
}


void
on_expander_mimms_output_activate      (GtkExpander     *expander,
                                        gpointer         user_data)
{

}


void
on_button_cancel_download_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	cancel_download();
}


void
on_button_close_download_window_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(lookup_widget(GTK_WIDGET(button), "download_status_window"));
}


gboolean
on_download_status_window_delete_event (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

  return FALSE;
}

